@@ -6,7 +6,7 @@
 #include"vroot.h"
 #endif
 #include "paint.h"
-char X_graphics_Id[]="$Id: X_graphics.c,v 1.22 1995/06/19 10:35:12 spb Exp spb $";
+char X_graphics_Id[]="$Id: X_graphics.c,v 1.24 2001/03/30 12:13:04 spb Exp $";
 
 char *display=NULL;       /* name of display to open, NULL for default */
 char *geom=NULL;          /* geometry of window, NULL for default */
@@ -45,6 +45,34 @@ Atom wm_delete_window;
 void zap_events();
 void finish_graphics();
   
+Graph g={
+1024,
+768,
+0,
+0.3,
+1.0,
+0.3,
+0.6,
+2.5,
+4.0,
+(40.0 * PI)/180.0,
+0.0,
+0.5,
+0.0,
+0.6,
+DEF_COL,
+60,
+10,
+2,
+FALSE,
+TRUE,
+20,
+0,
+0
+};
+
+
+
 /*{{{void zap_events(int snooze)*/
 void zap_events(snooze)
 int snooze;
@@ -53,10 +81,6 @@ int snooze;
   XExposeEvent *expose = (XExposeEvent *)&event;
   int exw, exh, i;
 
-#ifndef NO_SLEEP
-  i=0;
-  do{
-#endif
   while( XPending(dpy) ){
     XNextEvent(dpy, &event);
     switch(event.type) {
@@ -105,21 +129,6 @@ int snooze;
     finish_artist();
     exit(0);
   }
-#ifndef NO_SLEEP
-    /* sleeping is very bad because it will prevent
-     * events being processed but I suppose it is better
-     * than being a CPU hog, as a compremise always check for
-     * events at least once a second, looping for longer sleep times.
-     * process the events before a sleep to make sure the screen is up to date.
-     * the events must always be processed at least once.
-     */
-    if( snooze )
-    {
-      sleep(1);
-    }
-    i++;
-  }while( i<snooze );
-#endif
 }
 /*}}}*/
   
@@ -188,6 +197,14 @@ int lx,ly,ux,uy;
 }
 /*}}}*/
 
+/*{{{void blank_col( pos )*/
+void blank_col( pos )
+int pos;
+{
+  blank_region(pos,0,pos,graph_height);
+}
+/*}}}*/
+
 /*{{{void init_graphics( ... )*/
 void init_graphics( want_use_root, use_background, want_clear, s_graph_width,s_graph_height,ncol,red,green,blue )
 int want_use_root;    /* display on the root window */
@@ -215,6 +232,7 @@ Gun *blue;
   unsigned int border;
   unsigned long gcvmask;
   XGCValues gcv;
+  XWindowAttributes xgwa;
   
   int i;
   int newmap=FALSE;
@@ -239,7 +257,9 @@ Gun *blue;
   parent = RootWindow(dpy, screen);
 /*}}}*/
 /*{{{find appropriate vis*/
-  map=defaultmap=DefaultColormap(dpy,screen);
+  /* map=defaultmap=DefaultColormap(dpy,screen); */
+  XGetWindowAttributes (dpy, parent, &xgwa);
+  map=defaultmap=xgwa.colormap;
   vis = DefaultVisual(dpy,screen);
   depth = DefaultDepth(dpy,screen);
 /*}}}*/
